@@ -9,7 +9,10 @@ use ratatui::{
 };
 use tui_textarea::{CursorMove, TextArea};
 
-use crate::app::{keymap::Input, App};
+use crate::{
+    app::{keymap::Input, App},
+    settings::Settings,
+};
 
 use backend::{DataProvider, Entry};
 
@@ -23,17 +26,19 @@ mod tags;
 
 const FOOTER_TEXT: &str =
     "Enter or <Ctrl-m>: confirm | Esc or <Ctrl-c>: Cancel | Tab: Change focused control | <Ctrl-Space> or <Ctrl-t>: Open tags";
-const FOOTER_MARGINE: u16 = 15;
+const FOOTER_MARGIN: u16 = 15;
 
 pub struct EntryPopup<'a> {
     title_txt: TextArea<'a>,
     date_txt: TextArea<'a>,
     tags_txt: TextArea<'a>,
+    priority_txt: TextArea<'a>,
     is_edit_entry: bool,
     active_txt: ActiveText,
     title_err_msg: String,
     date_err_msg: String,
     tags_err_msg: String,
+    priority_err_msg: String,
     tags_popup: Option<TagsPopup>,
 }
 
@@ -42,18 +47,19 @@ enum ActiveText {
     Title,
     Date,
     Tags,
+    Priority,
 }
 
 #[derive(Debug, PartialEq, Eq)]
 pub enum EntryPopupInputReturn {
-    KeepPupup,
+    KeepPopup,
     Cancel,
     AddEntry(u32),
     UpdateCurrentEntry,
 }
 
 impl<'a> EntryPopup<'a> {
-    pub fn new_entry() -> Self {
+    pub fn new_entry(settings: &Settings) -> Self {
         let title_txt = TextArea::default();
 
         let date = Local::now();
@@ -67,15 +73,23 @@ impl<'a> EntryPopup<'a> {
 
         let tags_txt = TextArea::default();
 
+        let priority_txt = if let Some(priority) = settings.default_journal_priority {
+            TextArea::new(vec![priority.to_string()])
+        } else {
+            TextArea::default()
+        };
+
         Self {
             title_txt,
             date_txt,
             tags_txt,
+            priority_txt,
             is_edit_entry: false,
             active_txt: ActiveText::Title,
             title_err_msg: String::default(),
             date_err_msg: String::default(),
             tags_err_msg: String::default(),
+            priority_err_msg: String::default(),
             tags_popup: None,
         }
     }
@@ -96,29 +110,34 @@ impl<'a> EntryPopup<'a> {
         let mut tags_txt = TextArea::new(vec![tags]);
         tags_txt.move_cursor(CursorMove::End);
 
-        let mut entry_pupop = Self {
+        let prio = entry.priority.map(|pr| pr.to_string()).unwrap_or_default();
+
+        let mut priority_txt = TextArea::new(vec![prio]);
+        priority_txt.move_cursor(CursorMove::End);
+
+        let mut entry_popup = Self {
             title_txt,
             date_txt,
             tags_txt,
+            priority_txt,
             is_edit_entry: true,
             active_txt: ActiveText::Title,
             title_err_msg: String::default(),
             date_err_msg: String::default(),
             tags_err_msg: String::default(),
+            priority_err_msg: String::default(),
             tags_popup: None,
         };
 
-        entry_pupop.validate_title();
-        entry_pupop.validat_date();
-        entry_pupop.validat_tags();
+        entry_popup.validate_all();
 
-        entry_pupop
+        entry_popup
     }
 
     pub fn render_widget(&mut self, frame: &mut Frame, area: Rect) {
-        let mut area = centered_rect_exact_height(70, 14, area);
+        let mut area = centered_rect_exact_height(70, 17, area);
 
-        const FOOTER_LEN: u16 = FOOTER_TEXT.len() as u16 + FOOTER_MARGINE;
+        const FOOTER_LEN: u16 = FOOTER_TEXT.len() as u16 + FOOTER_MARGIN;
 
         if area.width < FOOTER_LEN {
             area.height += FOOTER_LEN / area.width;
@@ -144,6 +163,7 @@ impl<'a> EntryPopup<'a> {
                     Constraint::Length(3),
                     Constraint::Length(3),
                     Constraint::Length(3),
+                    Constraint::Length(3),
                     Constraint::Min(1),
                 ]
                 .as_ref(),
@@ -153,6 +173,7 @@ impl<'a> EntryPopup<'a> {
         self.title_txt.set_cursor_line_style(Style::default());
         self.date_txt.set_cursor_line_style(Style::default());
         self.tags_txt.set_cursor_line_style(Style::default());
+        self.priority_txt.set_cursor_line_style(Style::default());
 
         let active_cursor_style = Style::default().bg(Color::White).fg(Color::Black);
         let deactivate_cursor_style = Style::default().bg(Color::Reset);
@@ -162,16 +183,25 @@ impl<'a> EntryPopup<'a> {
                 self.title_txt.set_cursor_style(active_cursor_style);
                 self.date_txt.set_cursor_style(deactivate_cursor_style);
                 self.tags_txt.set_cursor_style(deactivate_cursor_style);
+                self.priority_txt.set_cursor_style(deactivate_cursor_style);
             }
             ActiveText::Date => {
                 self.title_txt.set_cursor_style(deactivate_cursor_style);
                 self.date_txt.set_cursor_style(active_cursor_style);
                 self.tags_txt.set_cursor_style(deactivate_cursor_style);
+                self.priority_txt.set_cursor_style(deactivate_cursor_style);
             }
             ActiveText::Tags => {
                 self.title_txt.set_cursor_style(deactivate_cursor_style);
                 self.date_txt.set_cursor_style(deactivate_cursor_style);
                 self.tags_txt.set_cursor_style(active_cursor_style);
+                self.priority_txt.set_cursor_style(deactivate_cursor_style);
+            }
+            ActiveText::Priority => {
+                self.title_txt.set_cursor_style(deactivate_cursor_style);
+                self.date_txt.set_cursor_style(deactivate_cursor_style);
+                self.tags_txt.set_cursor_style(deactivate_cursor_style);
+                self.priority_txt.set_cursor_style(active_cursor_style);
             }
         };
 
@@ -237,9 +267,28 @@ impl<'a> EntryPopup<'a> {
             );
         }
 
+        if self.priority_err_msg.is_empty() {
+            self.priority_txt.set_style(active_style);
+            self.priority_txt.set_block(
+                Block::default()
+                    .borders(Borders::ALL)
+                    .style(active_style)
+                    .title("Priority"),
+            );
+        } else {
+            self.priority_txt.set_style(invalid_style);
+            self.priority_txt.set_block(
+                Block::default()
+                    .borders(Borders::ALL)
+                    .style(invalid_style)
+                    .title(format!("Priority : {}", self.priority_err_msg)),
+            );
+        }
+
         frame.render_widget(self.title_txt.widget(), chunks[0]);
         frame.render_widget(self.date_txt.widget(), chunks[1]);
-        frame.render_widget(self.tags_txt.widget(), chunks[2]);
+        frame.render_widget(self.priority_txt.widget(), chunks[2]);
+        frame.render_widget(self.tags_txt.widget(), chunks[3]);
 
         let footer = Paragraph::new(FOOTER_TEXT)
             .alignment(Alignment::Center)
@@ -250,7 +299,7 @@ impl<'a> EntryPopup<'a> {
                     .style(Style::default()),
             );
 
-        frame.render_widget(footer, chunks[3]);
+        frame.render_widget(footer, chunks[4]);
 
         if let Some(tags_popup) = self.tags_popup.as_mut() {
             tags_popup.render_widget(frame, area)
@@ -261,12 +310,14 @@ impl<'a> EntryPopup<'a> {
         self.title_err_msg.is_empty()
             && self.date_err_msg.is_empty()
             && self.tags_err_msg.is_empty()
+            && self.priority_err_msg.is_empty()
     }
 
     pub fn validate_all(&mut self) {
         self.validate_title();
-        self.validat_date();
-        self.validat_tags();
+        self.validate_date();
+        self.validate_tags();
+        self.validate_priority();
     }
 
     fn validate_title(&mut self) {
@@ -277,7 +328,7 @@ impl<'a> EntryPopup<'a> {
         }
     }
 
-    fn validat_date(&mut self) {
+    fn validate_date(&mut self) {
         if let Err(err) = NaiveDate::parse_from_str(self.date_txt.lines()[0].as_str(), "%d-%m-%Y") {
             self.date_err_msg = err.to_string();
         } else {
@@ -285,7 +336,7 @@ impl<'a> EntryPopup<'a> {
         }
     }
 
-    fn validat_tags(&mut self) {
+    fn validate_tags(&mut self) {
         let tags = text_to_tags(
             self.tags_txt
                 .lines()
@@ -299,6 +350,15 @@ impl<'a> EntryPopup<'a> {
         }
     }
 
+    fn validate_priority(&mut self) {
+        let prio_text = self.priority_txt.lines().first().unwrap();
+        if !prio_text.is_empty() && prio_text.parse::<u32>().is_err() {
+            self.priority_err_msg = String::from("Priority must be a positive number");
+        } else {
+            self.priority_err_msg.clear();
+        }
+    }
+
     pub async fn handle_input<D: DataProvider>(
         &mut self,
         input: &Input,
@@ -307,7 +367,7 @@ impl<'a> EntryPopup<'a> {
         if self.tags_popup.is_some() {
             self.handle_tags_popup_input(input);
 
-            return Ok(EntryPopupInputReturn::KeepPupup);
+            return Ok(EntryPopupInputReturn::KeepPopup);
         }
 
         let has_ctrl = input.modifiers.contains(KeyModifiers::CONTROL);
@@ -319,18 +379,20 @@ impl<'a> EntryPopup<'a> {
             KeyCode::Tab | KeyCode::Down => {
                 self.active_txt = match self.active_txt {
                     ActiveText::Title => ActiveText::Date,
-                    ActiveText::Date => ActiveText::Tags,
+                    ActiveText::Date => ActiveText::Priority,
+                    ActiveText::Priority => ActiveText::Tags,
                     ActiveText::Tags => ActiveText::Title,
                 };
-                Ok(EntryPopupInputReturn::KeepPupup)
+                Ok(EntryPopupInputReturn::KeepPopup)
             }
             KeyCode::Up => {
                 self.active_txt = match self.active_txt {
                     ActiveText::Title => ActiveText::Tags,
                     ActiveText::Date => ActiveText::Title,
-                    ActiveText::Tags => ActiveText::Date,
+                    ActiveText::Priority => ActiveText::Date,
+                    ActiveText::Tags => ActiveText::Priority,
                 };
-                Ok(EntryPopupInputReturn::KeepPupup)
+                Ok(EntryPopupInputReturn::KeepPopup)
             }
             KeyCode::Char(' ') | KeyCode::Char('t') if has_ctrl => {
                 debug_assert!(self.tags_popup.is_none());
@@ -340,11 +402,11 @@ impl<'a> EntryPopup<'a> {
                     .tags_txt
                     .lines()
                     .first()
-                    .expect("Tags textbox has one line");
+                    .expect("Tags text box has one line");
 
                 self.tags_popup = Some(TagsPopup::new(tags_text, tags));
 
-                Ok(EntryPopupInputReturn::KeepPupup)
+                Ok(EntryPopupInputReturn::KeepPopup)
             }
             _ => {
                 match self.active_txt {
@@ -355,16 +417,21 @@ impl<'a> EntryPopup<'a> {
                     }
                     ActiveText::Date => {
                         if self.date_txt.input(KeyEvent::from(input)) {
-                            self.validat_date();
+                            self.validate_date();
                         }
                     }
                     ActiveText::Tags => {
                         if self.tags_txt.input(KeyEvent::from(input)) {
-                            self.validat_tags();
+                            self.validate_tags();
+                        }
+                    }
+                    ActiveText::Priority => {
+                        if self.priority_txt.input(KeyEvent::from(input)) {
+                            self.validate_priority();
                         }
                     }
                 }
-                Ok(EntryPopupInputReturn::KeepPupup)
+                Ok(EntryPopupInputReturn::KeepPopup)
             }
         }
     }
@@ -394,7 +461,7 @@ impl<'a> EntryPopup<'a> {
         // Validation
         self.validate_all();
         if !self.is_input_valid() {
-            return Ok(EntryPopupInputReturn::KeepPupup);
+            return Ok(EntryPopupInputReturn::KeepPopup);
         }
 
         let title = self.title_txt.lines()[0].to_owned();
@@ -412,22 +479,26 @@ impl<'a> EntryPopup<'a> {
                 .expect("Tags TextBox have one line"),
         );
 
+        let priority = match self.priority_txt.lines().first().unwrap() {
+            num if num.is_empty() => None,
+            num => Some(num.parse().expect("Priority must be validated before")),
+        };
+
         if self.is_edit_entry {
-            app.update_current_entry(title, date, tags).await?;
+            app.update_current_entry(title, date, tags, priority)
+                .await?;
             Ok(EntryPopupInputReturn::UpdateCurrentEntry)
         } else {
-            let entry_id = app.add_entry(title, date, tags).await?;
+            let entry_id = app.add_entry(title, date, tags, priority).await?;
             Ok(EntryPopupInputReturn::AddEntry(entry_id))
         }
     }
 }
 
-#[inline]
 fn tags_to_text(tags: &[String]) -> String {
     tags.join(", ")
 }
 
-#[inline]
 fn text_to_tags(text: &str) -> Vec<String> {
     text.split_terminator(',')
         .map(|tag| String::from(tag.trim()))
